/***************************************************************************
 *   Copyright (C) 2005 by Christoph Hormann  <chris_hormann@gmx.de>       *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#define KIO_POV_NAME "kio-pov"

#include <kinstance.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>

#include <kstandarddirs.h>
#include <kprocess.h>

#include <qstring.h>
#include <qcstring.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qdatastream.h>
#include <qregexp.h>

#include "povexe.moc"

/* --------------------------------------------------------------------------- */

POVExecutable::POVExecutable(const QString &name)
{
	id = name;

	// find out the tmp dir
	char *s;
	if ((s = getenv("TMPDIR"))) TmpDir = QString(s);
	else TmpDir = QString("/tmp");

	exe_path = KStandardDirs::findExe(id);
	info_generated = false;
}

/* --------------------------------------------------------------------------- */

bool POVExecutable::exists(void)
{
	if (exe_path == QString::null) return false;
	return true;
}

/* --------------------------------------------------------------------------- */

QString POVExecutable::ini(void)
{
	QString HomeDir;
	char *s;
	if ((s = getenv("HOME"))) HomeDir = QString(s);
	else return QString::null;

	if (!info_generated) TestRun();

	// determine main POV-Ray ini file that is used by POV-Ray
	// see section 1.5.2 of the POV-Ray for UNIX docs
	if ((s = getenv("POVINI"))) return QString(s);
	else
	{
		QString ini = HomeDir + "/." + id + "/" + pov_version_string + "/povray.ini";
		if (KStandardDirs::exists(ini)) return ini;
		else
		{
			ini = "/usr/local/etc/" + id + "/" + pov_version_string + "/povray.ini";
			if (KStandardDirs::exists(ini)) return ini;
			else
			{
				ini = HomeDir + "/.povrayrc";
				if (KStandardDirs::exists(ini)) return ini;
				else
				{
					ini = "/usr/local/etc/povray.ini";
					if (KStandardDirs::exists(ini)) return ini;
					else return QString::null;
				}
			}
		}
	}
}

/* --------------------------------------------------------------------------- */

void POVExecutable::slotGetStdError(KProcess*, char *s, int len)
{
	ErrorStream += QString::fromLocal8Bit(s, len);
}

bool POVExecutable::TestRun(void)
{
	if (info_generated) return false;

	bool result = false;

	QString scene_name(TmpDir + "/" + KIO_POV_NAME + "/" + id + "_test.pov");

	if (!KStandardDirs::exists(TmpDir + "/" + KIO_POV_NAME))
		KStandardDirs::makeDir(TmpDir + "/" + KIO_POV_NAME);
		
	// write test scene
	QFile pov_file( scene_name );
	if ( pov_file.open( IO_WriteOnly ) )
	{
		QTextStream pov_stream( &pov_file );
		pov_stream << "sphere{0,1}\n";
		pov_file.close();

		KProcess PovrayKProcess;

		QString scene_param = "-i" + scene_name;

		PovrayKProcess << id << scene_param << "-f" << "-w1" << "-h1" << "-p";

		ErrorStream = QString::null;

		connect(&PovrayKProcess, SIGNAL(receivedStderr(KProcess *, char *, int)),
						 this, SLOT(slotGetStdError(KProcess *, char *, int)));

		PovrayKProcess.start(KProcess::Block, KProcess::All);
			
		if (!PovrayKProcess.normalExit())
		{
			version_info = i18n("there was an error testing %1 (1)").arg(id);
			result = true;
		}
		else
		{
			int Pos = ErrorStream.find("Persistence of Vision(tm) Ray Tracer");
			int Pos2 = ErrorStream.find("This is an");
			version_info = ErrorStream.mid(Pos, Pos2-Pos);
			version_info.replace('\n',"");
			compile_info = ErrorStream.mid(Pos2, ErrorStream.find("POV-Ray is based on DKBTrace")-Pos2);
			Pos = version_info.find("Version ");
			Pos2 = version_info.mid(Pos+8).find(QRegExp("[0123456789]"));
			pov_version_string = version_info.mid(Pos+8+Pos2, 3);
			pov_version = pov_version_string.toFloat();

			Pos = ErrorStream.find("Library paths:")+15;
			QString LibPaths_String = ErrorStream.mid(Pos, ErrorStream.find("Output Options")-Pos);
			LibPaths_String.replace(QRegExp("[ ]+/"), "/");
			LibPaths = QStringList::split( QRegExp("[\r\n][\r\n]*"), LibPaths_String );
			info_generated = true;
		}
	}
	else
	{
		version_info = i18n("there was an error testing %1 (2)").arg(id);
		result = true;
	}

	return result;
}
